#!moo verb @password --on $player

# pylint: disable=return-outside-function,undefined-variable

"""
This verb is a player command used to change the player's password.
"""

print("@password is not yet implemented.")
